
class FTNAddress:

	import string, types


	def __init__(self, addr=None):

		self.__dict__ = {
			'domain': None,
			'zone'  : None,
			'region': None,
			'net'   : None,
			'hub'   : None,
			'node'  : None,
			'point' : None,
		}

		if addr:
			self.ParseAddr(addr)


	def __getattr__(self, attr):

		if attr in self.__dict__:
			return self.__dict__[attr]
		else:
			raise AttributeError('%s instance has no attribute \'%s\'' % (self.__class__, attr))


	def __setattr__(self, attr, value):

		if attr == 'domain':
			if value in ('', '*', None):
				self.__dict__[attr] = None
			else:
				for x in value:
					if not x in self.string.ascii_letters + self.string.digits:
						raise ValueError('%s instance attribute \'%s\' must be an alphanumeric string' % (self.__class__, attr))

				self.__dict__[attr] = value

		elif attr in ('zone', 'region', 'net', 'hub', 'node', 'point'):
			if value in ('', '*', None):
				self.__dict__[attr] = None
			else:
				try:
					self.__dict__[attr] = int(value)
				except:
					raise ValueError('%s instance attribute \'%s\' must be an integer' % (self.__class__, attr))
		else:
			raise AttributeError('%s instance has no attribute \'%s\'' % (self.__class__, attr))


	def __eq__(self, other):

		if not isinstance(other, FTNAddress):
			return False

		for element in self.__dict__:
			if not ((self.__dict__[element] is None) or \
			   (other.__dict__[element] is None) or \
			   (self.__dict__[element] == other.__dict__[element])):
				return False

		return True


	def __str__(self):

		s = str(self.node)

		if self.net:
			s = '%s/%s' % (self.net, s)

		if self.point:
			s = '%s.%s' % (s, self.point)

		if self.zone:
			s = '%s:%s' % (self.zone, s)

		if self.domain:
			s = '%s#%s' % (self.domain, s)

		return s


	def __repr__(self):

		a = [self.domain, self.zone, self.region, self.net, self.hub, self.node, self.point]

		for x in range(0, len(a)):
			if a[x] is None:
				a[x] = ''
			elif type(a[x]) == self.types.IntType:
				a[x] = str(a[x])

		return ':'.join(a)


	def ParseAddr(self, addr):

		self.__init__(None)

		if not type(addr) in self.types.StringTypes:
			raise TypeError('address must be a string type')

		x = addr.count(':')

		if x < 2:
			self._parseStandard(addr)
		elif x == 6:
			self._parseRLF(addr)
		else:
			raise ValueError('address must be in standard or routelist format')


	def _parseRLF(self, addr):

		parts = addr.split(':')

		for x in range(0, len(parts)):
			if parts[x] == '':
				parts[x] = None

		self.domain, self.zone, self.region, self.net, \
		self.hub, self.node, self.point = parts


	def _parseStandard(self, addr):

		if addr.find('@') > -1:
			self.domain = addr[addr.find('@')+1:]
			addr = addr[:addr.find('@')]

		if addr.find('#') > -1:
			self.domain = addr[:addr.find('#')]
			addr = addr[addr.find('#')+1:]

		if addr.find(':') > -1:
			self.zone = addr[:addr.find(':')]
			addr = addr[addr.find(':')+1:]

		if addr.find('.') > -1:
			self.point = addr[addr.find('.')+1:]
			addr = addr[:addr.find('.')]
		else:
			self.point = 0	# BUGBUG

		if addr.find('/') > -1:
			self.net = addr[:addr.find('/')]
			addr = addr[addr.find('/')+1:]

		if addr != '*':	# BUGBUG
			self.node = addr


	def casNode(self, node):

		self.node = node
		self.point = 0


	def casHub(self, hub):

		self.hub = hub
		self.casNode(hub)


	def casNet(self, net):

		self.net = net
		self.casHub(0)


	def casRegion(self, region):

		self.region = region
		self.casNet(region)


	def casZone(self, zone):

		self.zone = zone
		self.casRegion(zone)


	def casDomain(self, domain, default_zone = 1):

		self.domain = domain
		self.casZone(default_zone)
